/************************************************************************
 *																		*
 *	H8 EMULATOR Project (common) --	MSGS.H								*
 *																		*
 *	Named Pipe-based Interprocess Communication definitions for the		*
 *  Heathkit H-8 simulator project										*
 *																		*
 *	msgs.h: defines the inter-application message set, message syntax	*
 *	and names for the pipes.											*
 *																		*
 *	------------------------------------------------------------------	*
 *																		*
 *	Copyright (c) 2001 by			David A. Wallace					*
 *									154 Dunstable Rd.					*
 *									N. Chelmsford, MA					*
 *											01863-1202					*
 *																		*
 ************************************************************************/


#ifndef _MSGS_H_
#define _MSGS_H_

#ifndef __AFXTEMPL_H__
#include <afxtempl.h>
#endif


//	============================================================================
//
//	Message syntax
//
//	All messages are ASCII text ending in linefeed (0x0A).  Messages are
//	variable length semantically; it is permissable to "pad" the message to a
//	fixed size, since the linefeed character marks the end of the message.
//	Except for the linefeed character, messages consist of characters for which
//	isprint() would return TRUE.
//
//	Messages begin with a command code.  The command code is always a textual
//	representation of an unsigned short integer.  The command code is always
//	followed by a single space character if there are arguments or by a linefeed
//	if there are not.
//
//	If the command code is followed by arguments, argument parsing is performed
//	as a series of space-separated (possibly quoted) strings until the linefeed
//	character is recognized.  Semantic content of the argument list is between
//	the originating and receiving applications to negotiate -- the message layer
//	simply treats a message as "<command_code>[<space><arg>]...<linefeed>".
//
//	Two functions are provided to assemble and parse messages:
//
//	CString	make_message(int nCommandCode, CStringList cslArgs);
//	BOOL	parse_message(CString strMessage,
//							int & nCommandCode, CStringList & cslArgs);
//
//	make_message will perform any necessary quoting and quote-escape operations
//	on the arguments; parse_message will undo these adjustments.  If the
//	parse_message function returns FALSE, the message has a syntax fault.  The
//	use of the CStringList object to represent the argument list provides a
//	convenient package for argument enumerating and counting, allowing commands
//	to have varying number of arguments as desired.
//

CString	make_message(int nCommandCode, CStringList & cslArgs);
BOOL	parse_message(CString strMessage,
						int & nCommandCode, CStringList & cslArgs);

#define	MAX_MESSAGE_LENGTH	 1024	//	up to 1024 characters in a message.
#define	MAX_ARG_COUNT			2	//	most args in any message yet defined.

//	----------------------------------------------------------------------------
//
//	Pipe Names
//
//	The general rule for naming of pipes is APP_NAME.RX and APP_NAME.TX, where
//	APP_NAME is the name of the slave application.  APP_NAME.RX is the pipe that
//	the slave expects to receive messages on from the master; APP_NAME.TX is the
//	pipe that the master expects to receive messages on from the slave.  This
//	naming convention needs to be adjusted for situations where a given
//	application is slave to more than one master:  in that situation -- which
//	has not yet happened, by the way -- the name would need to reflect both ends
//	of the connection, such as "SLAVE_AND_MASTER_APPS.RX".  (The use of "FROM"
//	and "TO" should be avoided in the name, as these words imply a direction for
//	the data transfer which is, of course, dependent on which side of the
//	connection you sit when you use it.)

#define	CASSETTE_RX		"\\\\.\\PIPE\\tape.rx"
#define	CASSETTE_TX		"\\\\.\\PIPE\\tape.tx"
#define	H17_RX			"\\\\.\\PIPE\\h17.rx"
#define H17_TX			"\\\\.\\PIPE\\h17.tx"

enum {
	CASSETTE_CONNECTION = 0,
	H17_CONNECTION,
	MAX_CONNECTIONS			//	must be last enum
	};
//	----------------------------------------------------------------------------
//
//	Types, structures and defines for managing the pipe communication interface:
//

typedef enum {
	PIPESTATE_CLOSED = 0,
	PIPESTATE_IDLE,
	PIPESTATE_XFER,
	PIPESTATE_READY,
	PIPESTATE_BROKEN
} PIPE_IO_STATE;

typedef enum {
	PIPEMODE_UNDEFINED = 0,
	PIPEMODE_SERVER_READ,
	PIPEMODE_SERVER_WRITE,
	PIPEMODE_CLIENT_READ,
	PIPEMODE_CLIENT_WRITE,
	PIPEMODE_SERVER_READ_OVERLAPPED,
	PIPEMODE_SERVER_WRITE_OVERLAPPED,
	PIPEMODE_CLIENT_READ_OVERLAPPED,
	PIPEMODE_CLIENT_WRITE_OVERLAPPED
} PIPE_MODE;

struct PIPE_CONTROL {
	DWORD			dwBufferLength;
	DWORD			dwXferCount;		// Size of actual data transfer
	HANDLE			hPipe;				// Handle of pipe
	PIPE_MODE		ePipeMode;			// Pipe modality
	PIPE_IO_STATE	ePipeState;			// State of current I/O operation
	OVERLAPPED		overControl;		// Overlapped struct that controls
	char 			szBuffer[
						MAX_MESSAGE_LENGTH + 1];	// Transfer buffer
};

#define PIPE_DEFAULT_TIMEOUT			10000
#define PIPE_SAMPLING_RATE				100	


//	----------------------------------------------------------------------------
//
//	Definitions of inter-application messages
//
//	General notes:
//		1.	Applications communicate over a full-duplex link that uses a pair of
//			named pipes.
//		2.	A "master" application is the one which created the pair of pipes;
//			the "slave" application is on the other end of the connection.
//		3.	Any message whose command code is less than 1024 is a realtime
//			message -- the recipient is expected to act on the message as soon
//			as it is received.  Messages with command codes from 1024 up are
//			queued messages -- the recipient is free to store these messages in
//			a FIFO list and process them as time permits.

const int  SHUTDOWN_NOW_CMD                     = 0;
//	Message Syntax:	"SHUTDOWN_NOW_CMD"
//  Type:			REALTIME
//	Issued By:		any application
//  To:				any application
//	Semantics:		requests that the target application terminate.
//	Response:		none expected, might get APP_IS_CLOSING if the target is a
//					slave app or the pipes may close if the target is a master
//					app.
//	Notes:			This is an emergency shut-down message, essentially a notice
//					that the originator has failed and must exit.


const int  APP_IS_CLOSING						= 1;
//	Message Syntax:	"APP_IS_CLOSING"
//  Type:			REALTIME
//	Issued By:		any "slave" application
//  To:				any "master" application
//	Semantics:		This message notifies the "master" application that the
//					"slave" application is closing.  It is a request that the
//					"master" disconnect the pipes that connect it to the slave.
//	Response:		pipes will close.
//	Notes:			1. The "slave" application should wait until the pipes it is
//					using have closed before exiting.  The "master" application
//					should, after noting that the slave has exited, re-create
//					the pipes so that a subsequent instance of the slave app
//					can make use of the channel.
//					2. The terms "master" and "slave" refer to who creates the
//					pipes of a communication channel.  A "master" application
//					is the one that creates the pipes.


const int	NORMAL_STOP							= 1024;
//	Message Syntax:	"NORMAL_STOP"
//	Type:			QUEUED
//	Issued By:		any "master" application
//	To:				any "slave" application
//	Semantics:		This message notifies the slave that the master application
//					would like to stop and is requesting that the connected
//					slave application do so as well.
//	Response:		APP_IS_CLOSING from the slave.
//	Notes:			The master application should send this message to all of
//					its slaves and wait for them all to respond before closing
//					their pipes and exiting.


const int	CASSETTE_CHANGE						= 1025;
//	Message Syntax:	"CASSETTE_CHANGE {0 | 1} [path]"
//	Type:			QUEUED
//	Issued By:		TAPEUI
//	To:				H8CTRL
//	Semantics:		CASSETTE_CHANGE 0 pertains to the Reader device;
//					CASSETTE_CHANGE 1 pertains to the Recorder device.  If no
//					path argument is supplied, the notification is that the
//					device has no cassette "mounted" (e.g.: at start-up or
//					when the operator presses EJECT on the device).
//	Response:		none directly, though a CASSETTE_LOCK will occur if the
//					emulator wants to do IO to the device.


const int	CASSETTE_LOCK						= 1026;
//	Message Syntax:	"CASSETTE_LOCK { 0 | 1} {0 | 1}"
//	Type:			QUEUED
//	Issued By:		H8CTRL
//	To:				TAPEUI
//	Semantics:		First argument is the device (0 = reader, 1 = recorder);
//					second argument is interpreted as a BOOL, TRUE means the
//					cassette eject is disabled; FALSE means the cassette eject
//					is enabled.
//	Response:		none.
//	Notes:			Cassettes are "locked" while the LOAD or DUMP operation is
//					in progress.  This prevents the user from attempting to
//					re-define the filename currently "mounted" on the device
//					while I/O is in progress.  An indicator will light on the
//					device when the cassette is locked.


const int	CONTEXT_INQUIRY						= 1027;
//	Message Syntax:	"CONTEXT_INQUIRY index"
//					where index is a number.  Currently-defined values are:
//					0 - The path for the H8Enviroment
//
//	Type:			QUEUED
//	Issued By:		any slave application
//	To:				H8CTRL
//	Semantics:		This message asks H8CTRL, the applciation which keeps all
//					such information, to return a context value.  Context values
//					are pieces of information (usually paths or configuration
//					settings) needed by other applications in order to
//					initialize or access data in a way consistent with the rest
//					of the virtual computer system.
//	Response:		CONTEXT_VALUE


const int	CONTEXT_VALUE						= 1028;
//	Message Syntax:	"CONTEXT_VALUE index value"
//					where index is a number.  Currently-defined values are:
//					0 - The path for the H8Enviroment
//
//					and value is the information requested
//	Type:			QUEUED
//	Issued By:		H8CTRL
//	To:				originator of CONTEXT_INQUIRY request
//	Semantics:		This message is the response to a request to return a
//					context value.
//	Response:		none



const int	DOOR_OPEN							= 1029;
//	Message Syntax:	"DOOR_OPEN unit"
//					where "unit" is the device unit number (0 thru 7)
//	Type:			QUEUED
//	Issued By:		H17
//	To:				H8CTRL
//	Semantics:		indicates that the door on the specified unit was open
//	Response:		none
//	Notes:			this should act to prevent sector pulses from being
//					detected when the software examines the state of that unit.

const int	DOOR_CLOSED							= 1030;
//	Message Syntax:	"DOOR_CLOSED unit path"
//					where "unit" is the device unit number (0 thru 7) and
//						"path" is the file simulating the media inserted.
//	Type:			REALTIME or QUEUED
//	Type:			QUEUED
//	Issued By:		H17
//	To:				H8CTRL
//	Semantics:		indicates that the door on the specified unit was closed and
//					that data to/from this unit should be written to/read from
//					the indicated file.
//	Response:		none
//	Notes:			this should act to permit sector pulses to be detected when
//					the software examines the state of that unit.  Device I/O
//					will map into reads and writes to the indicated file, with
//					sector header data being synthesized.

const int	UNIT_SELECT							= 1031;
//	Message Syntax:	"UNIT_SELECT unit select"
//					where "unit" is the device unit number (0 thru 7) and
//						"select" is a BOOL (TRUE = selected).
//	Type:			QUEUED
//	Issued By:		H8CTRL
//	To:				H17
//	Semantics:		indicates that the specified unit is being read from
//					or written to.
//	Response:		none
//	Notes:			this should cause the H17 UI to change the state of
//					the specified unit's select indicator.



//	Prototype for message definition:

//	const int	command_name						= value;
//	Message Syntax:	"command_name args"
//					explaination of args, if needed.
//	Type:			REALTIME or QUEUED
//	Issued By:		originating app name
//	To:				target app name
//	Semantics:		what it do
//	Response:		expected reaction of target on receipt of the message
//	Notes:			other stuff


#endif	//	#ifndef _MSGS_H_